<?php

namespace Andy\SimilarThreads\Repository;

use XF\Mvc\Entity\Repository;

class SimilarThreads extends Repository
{
	public function getSimilarThreads($currentNodeId, $currentThreadId, $threadTitle)
	{
		//########################################
		// get similar threads
		//########################################

		// get options
		$options = \XF::options();	
		
		// get options from Admin CP -> Options -> Similar threads -> Exclude days
		$excludeDays = $options->similarThreadsExcludeDays;

		// get options from Admin CP -> Options -> Similar threads -> Exclude forums
		$excludeForums = $options->similarThreadsExcludeForums;
		
		// get options from Admin CP -> Options -> Similar threads -> Miniumum common word length
		$minimumCommonWordLength = $options->similarThreadsMinimumCommonWordLength;
		
		// get options from Admin CP -> Options -> Similar threads -> Limit
		$limit = $options->similarThreadsLimit;
		
		// get options from Admin CP -> Options -> Similar threads -> Multibyte
		$multibyte = $options->similarThreadsMultibyte;
		
		// get options from Admin CP -> Options -> Similar threads -> Same forum
		$sameForum = $options->similarThreadsSameForum;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop words
		$stopWords = $options->similarThreadsStopWords;
        
		//########################################
		// define variables
		//########################################

		$searchWords = array();
		$searchWord1 = '';
		$searchWord2 = '';
		$searchWord3 = '';
		$results1 = array();
		$results2 = array();
		$results3 = array();
		$results4 = array();
		$results5 = array();
		
		//########################################
		// return if currentNodeId is excluded
		//########################################

		// check for excluded forums
		if (in_array($currentNodeId, $excludeForums))
		{
			return;
		}

		//########################################
		// prepare search words
		//########################################

        // check condition
		if (!$multibyte)
		{
            $threadTitle = strtolower($threadTitle);
            $threadTitle = preg_replace('/[\p{P}]/u', ' ', $threadTitle);
            $threadTitle = str_replace('+', '', $threadTitle);
            $threadTitle = str_replace('|', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF0-\xF7].../', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF8-\xFB]..../', '', $threadTitle);
			$threadTitleArray = explode(' ', $threadTitle);
            $threadTitleArray = array_filter($threadTitleArray);
		}
	
        // check condition
		if ($multibyte)
		{
            $threadTitle = mb_strtolower($threadTitle);
            $threadTitle = preg_replace('/[\p{P}]/u', ' ', $threadTitle);
            $threadTitle = str_replace('+', '', $threadTitle);
            $threadTitle = str_replace('|', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF0-\xF7].../', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF8-\xFB]..../', '', $threadTitle);
			$threadTitleArray = mb_split(' ', $threadTitle);
            $threadTitleArray = array_filter($threadTitleArray);
		} 

		// create stopWordsArray
		$stopWordsArray = explode(' ', $stopWords);	
 		
		// remove words from array
		foreach ($threadTitleArray as $var)
		{
			if (!$multibyte)
			{
				if (!in_array(strtolower($var), $stopWordsArray))
				{		
					// verify minimumCommonWordLength			
					if (strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}

			if ($multibyte)
			{
				if (!in_array(mb_strtolower($var), $stopWordsArray))
				{	
					// verify minimumCommonWordLength				
					if (mb_strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}
		}

		// get count
		$count = count($searchWords);

		// return if no searchWords
		if ($count == 0)
		{
			return;
		}			
		
		// continue we have a search word
		if ($count > 0)
		{				
			// get first search word
			$searchWord1 = $searchWords[0];
			
			if ($count > 1)
			{	
				// get second search word
				$searchWord2 = $searchWords[1];	
			}
			
			if ($count > 2)
			{	
				// get third search word
				$searchWord3 = $searchWords[2];	
			}							
		}

		//########################################
		// exclude by age
		//########################################	
		
		if ($excludeDays > 0)
		{
			// convert to Unix timestamp
			$gte = time() - (86400 * $excludeDays);
		}
		else
		{
			$gte = 0;
		}
	
		//########################################
		// create conditions variable for nodes
		//########################################
		
		if (!$sameForum)
		{
			// get all viewable nodes
			$nodeRepo = $this->repository('XF:Node');
			$nodes = @$nodeRepo->getNodeList();
			$nodeIds = $nodes->keys();
			
			$arr = array_diff($nodeIds, $excludeForums);
			$result = array_values($arr);
			
			foreach ($result AS $k => $v)
			{
				if (\XF::visitor()->hasNodePermission($v, 'viewOthers'))
				{
					// get viewable node
					$conditionsOr[] = ['node_id', '=', $v];
				}
			}
		}
		
		if ($sameForum)
		{
			$currentNodeIdArray = $currentNodeId;
			
			// get viewable node
			$conditionsOr[] = ['node_id', '=', $currentNodeIdArray];
		}				

		//########################################
		// search 1
		//########################################

		if ($searchWord1 != '' AND $searchWord2 != '' AND $searchWord3 != '')
		{
			$limit1 = $limit;
            
            $finder = \XF::finder('XF:Thread');
			
			$mysqlVersion = \XF::app()->db()->getServerVersion();
			$mysqlVersion = substr($mysqlVersion, 0, 1);

			if ($mysqlVersion >= 8)
			{
				$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
				$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				$searchValue3 = '\'' . $finder->escapeExpression($searchWord3) . '\\\\b\'';
			}
			else
			{
				$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
				$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				$searchValue3 = '\'[[:<:]]' . $finder->escapeExpression($searchWord3) . '[[:>:]]\'';
			}
            
            $results1 = $finder
                ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue1))
                ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue2))
                ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue3))
				->where('discussion_state', '=', 'visible')
				->where('discussion_type', '<>', 'redirect')
				->where('post_date', '>=', $gte)
				->where('thread_id', '<>', $currentThreadId)
				->whereOr($conditionsOr)
				->order('post_date', 'DESC')
				->limit($limit1)
				->fetchColumns(['thread_id']);
		}

		//########################################
		// search 2
		//########################################		
		
		if ($searchWord1 != '' AND $searchWord2 != '')
		{				
			foreach ($results1 AS $k => $v)
			{
				$threadId = $v['thread_id'];
				$conditions[] = ['thread_id', '<>', $threadId];
			}
			
			if (empty($conditions))
			{
				$conditions = [];
			}

			$count = count($results1);	

			if ($count < $limit AND is_numeric($count))
			{
				$limit2 = $limit - $count;
                
                $finder = \XF::finder('XF:Thread');
				
				$mysqlVersion = \XF::app()->db()->getServerVersion();
				$mysqlVersion = substr($mysqlVersion, 0, 1);

				if ($mysqlVersion >= 8)
				{
					$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
					$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				}
				else
				{
					$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
					$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				}

                $results2 = $finder
                    ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue1))
                    ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue2))
                    ->where('discussion_state', '=', 'visible')
                    ->where('discussion_type', '<>', 'redirect')
                    ->where('post_date', '>=', $gte)
                    ->where('thread_id', '<>', $currentThreadId)
                    ->whereOr($conditionsOr)
                    ->where($conditions)
                    ->order('post_date', 'DESC')
                    ->limit($limit2)
                    ->fetchColumns(['thread_id']);
			}
		}		

		//########################################
		// search 3
		//########################################

		if ($searchWord1 != '')
		{	
			foreach ($results2 AS $k => $v)
			{
				$threadId = $v['thread_id'];
				$conditions[] = ['thread_id', '<>', $threadId];
			}
			
			if (empty($conditions))
			{
				$conditions = [];
			}
			
			$count = count($results1) + count($results2);

			if ($count < $limit AND is_numeric($count))
			{
				$limit3 = $limit - $count;

                $finder = \XF::finder('XF:Thread');

				$mysqlVersion = \XF::app()->db()->getServerVersion();
				$mysqlVersion = substr($mysqlVersion, 0, 1);

				if ($mysqlVersion >= 8)
				{
					$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
				}
				else
				{
					$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
				}

                $results3 = $finder
                    ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue1))
                    ->where('discussion_state', '=', 'visible')
                    ->where('discussion_type', '<>', 'redirect')
                    ->where('post_date', '>=', $gte)
                    ->where('thread_id', '<>', $currentThreadId)
                    ->whereOr($conditionsOr)
                    ->where($conditions)
                    ->order('post_date', 'DESC')
                    ->limit($limit3)
                    ->fetchColumns(['thread_id']);
			}
		}

		//########################################
		// search 4
		//########################################
		
		if ($searchWord2 != '')
		{
			foreach ($results3 AS $k => $v)
			{
				$threadId = $v['thread_id'];
				$conditions[] = ['thread_id', '<>', $threadId];
			}
			
			if (empty($conditions))
			{
				$conditions = [];
			}
			
			$count = count($results1) + count($results2) + count($results3);

			if ($count < $limit AND is_numeric($count))
			{
				$limit4 = $limit - $count;
				
                $finder = \XF::finder('XF:Thread');

				$mysqlVersion = \XF::app()->db()->getServerVersion();
				$mysqlVersion = substr($mysqlVersion, 0, 1);

				if ($mysqlVersion >= 8)
				{
					$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				}
				else
				{
					$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				}

                $results4 = $finder
                    ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue2))
                    ->where('discussion_state', '=', 'visible')
                    ->where('discussion_type', '<>', 'redirect')
                    ->where('post_date', '>=', $gte)
                    ->where('thread_id', '<>', $currentThreadId)
                    ->whereOr($conditionsOr)
                    ->where($conditions)
                    ->order('post_date', 'DESC')
                    ->limit($limit4)
                    ->fetchColumns(['thread_id']);
			} 
		}
		
		//########################################
		// search 5
		//########################################
		
		if ($searchWord3 != '')
		{
			foreach ($results4 AS $k => $v)
			{
				$threadId = $v['thread_id'];
				$conditions[] = ['thread_id', '<>', $threadId];
			}
			
			if (empty($conditions))
			{
				$conditions = [];
			}
			
			$count = count($results1) + count($results2) + count($results3) + count($results4);

			if ($count < $limit AND is_numeric($count))
			{
				$limit5 = $limit - $count;
				
                $finder = \XF::finder('XF:Thread');

				$mysqlVersion = \XF::app()->db()->getServerVersion();
				$mysqlVersion = substr($mysqlVersion, 0, 1);

				if ($mysqlVersion >= 8)
				{
					$searchValue3 = '\'' . $finder->escapeExpression($searchWord3) . '\\\\b\'';
				}
				else
				{
					$searchValue3 = '\'[[:<:]]' . $finder->escapeExpression($searchWord3) . '[[:>:]]\'';
				}

                $results5 = $finder
                    ->where($finder->expression('xf_thread.title RLIKE ' . $searchValue3))
                    ->where('discussion_state', '=', 'visible')
                    ->where('discussion_type', '<>', 'redirect')
                    ->where('post_date', '>=', $gte)
                    ->where('thread_id', '<>', $currentThreadId)
                    ->whereOr($conditionsOr)
                    ->where($conditions)
                    ->order('post_date', 'DESC')
                    ->limit($limit5)
                    ->fetchColumns(['thread_id']);
			} 
		}	

		//########################################
		// prepare similarThreads
		//########################################
        
        // get results
        $results = array_merge($results1, $results2, $results3, $results4, $results5);

		// return if empty
		if (empty($results))
		{
			return;
		}
        
        // foreach condition
        foreach ($results AS $result)
        {
            $ids[] = $result['thread_id'];
        }

        // get similarThreads
        $finder = \XF::finder('XF:Thread')->whereIds($ids);
        $similarThreads = $finder->fetch();
        $similarThreads = $similarThreads->sortByList($ids);

		// return similarThreads
		return $similarThreads;
	}
	
	public function getSimilarThreadsEnhanced($currentNodeId, $currentThreadId, $threadTitle)
	{
		//########################################
		// get similar threads enhanced
		//########################################

		// get options
		$options = \XF::options();
		
		// get options from Admin CP -> Options -> Similar threads -> Exclude days
		$excludeDays = $options->similarThreadsExcludeDays;

		// get options from Admin CP -> Options -> Similar threads -> Exclude forums
		$excludeForums = $options->similarThreadsExcludeForums;
		
		// get options from Admin CP -> Options -> Similar threads -> Limit
		$limit = $options->similarThreadsLimit;
		
		// get options from Admin CP -> Options -> Similar threads -> Miniumum common word length
		$minimumCommonWordLength = $options->similarThreadsMinimumCommonWordLength;
		
		// get options from Admin CP -> Options -> Similar threads -> Multibyte
		$multibyte = $options->similarThreadsMultibyte;
		
		// get options from Admin CP -> Options -> Similar threads -> Same forum
		$sameForum = $options->similarThreadsSameForum;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop words
		$stopWords = $options->similarThreadsStopWords;
		
		// get options from Admin CP -> Options -> Similar threads -> Error log
		$errorLog = $options->similarThreadsErrorLog;
		
		// get options from Admin CP -> Options -> Debug options -> xfesEnabled
		$xfesEnabled = @$options->xfesEnabled;
		
		// return error log
		if (!$xfesEnabled)
		{
			\XF::app()->error()->logError("Please deselect Enhanced search in Similar threads options page.");
			return;
		}

		//########################################
		// define variables
		//########################################
		
		// set other variables
		$searchWords = array();
		$searchWord1 = '';
		$searchWord2 = '';
		$searchWord3 = '';
		$excludeThreadIds = array();
		$continue = true;
		$threadIds = array();
		
		//########################################
		// return if currentNodeId is excluded
		//########################################

		// check for excluded forums
		if (in_array($currentNodeId, $excludeForums))
		{
			return;
		}

		//########################################
		// prepare search words
		//########################################
		
        // check condition
		if (!$multibyte)
		{
            $threadTitle = strtolower($threadTitle);
            $threadTitle = preg_replace('/[\p{P}]/u', ' ', $threadTitle);
            $threadTitle = str_replace('+', '', $threadTitle);
            $threadTitle = str_replace('|', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF0-\xF7].../', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF8-\xFB]..../', '', $threadTitle);
			$threadTitleArray = explode(' ', $threadTitle);
            $threadTitleArray = array_filter($threadTitleArray);
		}
		
        // check condition
		if ($multibyte)
		{
            $threadTitle = mb_strtolower($threadTitle);
            $threadTitle = preg_replace('/[\p{P}]/u', ' ', $threadTitle);
            $threadTitle = str_replace('+', '', $threadTitle);
            $threadTitle = str_replace('|', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF0-\xF7].../', '', $threadTitle);
			$threadTitle = preg_replace('/[\xF8-\xFB]..../', '', $threadTitle);
			$threadTitleArray = mb_split(' ', $threadTitle);
            $threadTitleArray = array_filter($threadTitleArray);
		} 
		
		// create stopWordsArray
		$stopWordsArray = explode(' ', $stopWords);								
 		
		// remove words from array
		foreach ($threadTitleArray as $var)
		{
			if (!$multibyte)
			{
				if (!in_array(strtolower($var), $stopWordsArray))
				{		
					// verify minimumCommonWordLength			
					if (strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}

			if ($multibyte)
			{
				if (!in_array(mb_strtolower($var), $stopWordsArray))
				{	
					// verify minimumCommonWordLength				
					if (mb_strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}
		}

		// get count
		$count = count($searchWords);

		// return in no searchWords
		if ($count == 0)
		{
			return;
		}			
		
		// continue we have a search word
		if ($count > 0)
		{				
			// get first search word
			$searchWord1 = $searchWords[0];
			
			if ($count > 1)
			{	
				// get second search word
				$searchWord2 = $searchWords[1];	
			}
			
			if ($count > 2)
			{	
				// get third search word
				$searchWord3 = $searchWords[2];	
			}							
		}

		//########################################
		// exclude by age
		//########################################	
		
		if ($excludeDays > 0)
		{
			// convert to Unix timestamp
			$gte = time() - (86400 * $excludeDays);
		}
		else
		{
			$gte = 0;
		}

		//########################################
		// get eleasticsearch information
		//########################################
		
		// get configArray
		$configArray = $options->xfesConfig;

		// get host	
		$host = $configArray['host'];			

		// get port
		$port = $configArray['port'];
		
		// get https
		$https = $configArray['https'];

		// get index
		$index = $configArray['index'];
		
        // check condition
        if ($https)
        {
            $url = 'https://' . $host . ':' . $port . '/' . $index . '/_search';
        }
        else
        {
            $url = 'http://' . $host . ':' . $port . '/' . $index . '/_search';
        }

		//########################################
		// get excludeNodeIds
		//########################################
		
		if (!empty($excludeForums))
		{
			foreach ($excludeForums as $value)
			{
				$excludeNodeIds[] = $value;
			}
		}
		
		if (empty($excludeForums))
		{
			$excludeNodeIds[] = 0;
		}
		
		//########################################
		// create conditions variable for nodes
		//########################################
		
		if (!$sameForum)
		{
			// get all viewable nodes
			$nodeRepo = $this->repository('XF:Node');
			$nodes = $nodeRepo->getNodeList();
			$nodeIds = $nodes->keys();
			
			$arr = array_diff($nodeIds, $excludeForums);
			$result = array_values($arr);
			
			foreach ($result AS $k => $v)
			{
				if (\XF::visitor()->hasNodePermission($v, 'viewOthers'))
				{
					// get viewable node
					$conditionsOr[] = $v;
				}
			}
		}
		
		if ($sameForum)
		{
			// get viewable node
			$conditionsOr[] = $currentNodeId;
		}

		//########################################
		// search 1
		//########################################

		if ($continue AND $searchWord1 AND $searchWord2 AND $searchWord3)
		{
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2 . ' ' . $searchWord3,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"term" => array(
												"discussion_id" => $currentThreadId
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl =  
				array(
					"from" => 0, "size" => $limit,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2 . ' ' . $searchWord3,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										),
										array(
											"terms" => array(
												"node" => $conditionsOr
											)
										)
									],
									"must_not" => [
										array(
											"term" => array(
												"discussion_id" => $currentThreadId
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'exceptions' => false,
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ]
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
					if ($errorLog)
					{
                    	\XF::logException($e, false, "Similar threads error: ");
					}
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }
		
		//########################################
		// search 2
		//########################################

		if ($continue AND $searchWord1 AND $searchWord2)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit2 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit2,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit2,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										),
										array(
											"terms" => array(
												"node" => $conditionsOr
											)
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                      
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'exceptions' => false,
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ]
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    if ($errorLog)
					{
                    	\XF::logException($e, false, "Similar threads error: ");
					}
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }

		//########################################
		// search 3
		//########################################

		if ($continue AND $searchWord1)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit3 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit3,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit3,
					"query" => array(
						"bool" => array(						
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										),
										array(
											"terms" => array(
												"node" => $conditionsOr
											)
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									] 
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}

			// get json
			$json = json_encode($dsl);
   
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'exceptions' => false,
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ]
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    if ($errorLog)
					{
                    	\XF::logException($e, false, "Similar threads error: ");
					}
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }			

		//########################################
		// search 4
		//########################################

		if ($continue AND $searchWord2)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit4 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit4,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord2
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit4,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord2
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										),
										array(
											"terms" => array(
												"node" => $conditionsOr
											)
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                        
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'exceptions' => false,
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ]
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    if ($errorLog)
					{
                    	\XF::logException($e, false, "Similar threads error: ");
					}
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }

		//########################################
		// search 5
		//########################################

		if ($continue AND $searchWord3)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit5 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit5,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord3
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                      
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit5,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord3
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										),
										array(
											"terms" => array(
												"node" => $conditionsOr
											)
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
		
			// get json
			$json = json_encode($dsl);

            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'exceptions' => false,
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ]
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    if ($errorLog)
					{
                    	\XF::logException($e, false, "Similar threads error: ");
					}
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }
		}

		//########################################
		// prepare similarThreads
		//########################################

		// return if empty
		if (empty($threadIds))
		{
			return;
		}

        // get similarThreads
        $finder = \XF::finder('XF:Thread')->whereIds($threadIds);
        $similarThreads = $finder->fetch();
        $similarThreads = $similarThreads->sortByList($threadIds);

		// return similarThreads
		return $similarThreads;
	}
}